@@ -19,6 +19,8 @@ import yagmail
 import smtplib
 from dingtalkchatbot.chatbot import DingtalkChatbot
 
+class AuthError(Exception):
+    pass
 
 def mac_notify(title,text):
     """
@@ -40,13 +42,20 @@ class Notifier(object):
     __metaclass__ = ABCMeta
 
     def __init__(self, *args, **kwargs):
-        pass
+        self._check_configure()
 
 
     @abstractmethod
     def send_text(self, text):
         pass
 
+    @abstractmethod
+    def _check_configure(self):
+        '''
+        验证基本配置信息是否正确
+        :return:
+        '''
+        pass
 
 class ServerChan(Notifier):
     """
@@ -54,9 +63,10 @@ class ServerChan(Notifier):
     """
 
     def __init__(self, token=None, *args, **kwargs):
-        super(ServerChan, self).__init__(*args, **kwargs)
         self._token = token
-        self.__api = "https://sc.ftqq.com/{}.send?text={}"
+        self.__api = "https://sc.ftqq.com/{token}.send?text={text}&desp={desp}"
+        self.__new_api = "https://sctapi.ftqq.com/{token}.send?title={title}&desp={desp}"
+        super(ServerChan, self).__init__(*args, **kwargs)
 
 
     @property
@@ -68,23 +78,75 @@ class ServerChan(Notifier):
     def token(self, token):
         self._token = token
 
+    def _check_configure(self):
+        if not self._token:
+            raise AuthError('请设置ServerChan token')
 
-    def send_text(self, text):
+    def send_text(self, text="", title="", desp=""):
         '''
-        请求结果：
+        老接口(即将下线)请求结果：
             成功：{'dataset': 'done', 'errmsg': 'success', 'errno': 0}
             短时间内重复发送：{'errmsg': '不要重复发送同样的内容', 'errno': 1024}
             失败：{"errno":1024,"errmsg":"bad pushtoken"}
-        :param text:
+
+        新接口请求结果：
+            成功：
+                        {
+              "code": 0,
+              "message": "",
+              "data": {
+                "pushid": "1663686",
+                "readkey": "SCTPmUPAz9ljk3Q",
+                "error": "SUCCESS",
+                "errno": 0
+              }
+            }
+            失败：
+                空title
+                {
+                  "message": "[INPUT]title 不能为空",
+                  "code": 20001,
+                  "info": "title 不能为空",
+                  "args": [
+                    null
+                  ]
+                }
+                错误token
+                {
+                  "message": "[AUTH]用户不存在或者权限不足",
+                  "code": 40001,
+                  "info": "用户不存在或者权限不足",
+                  "args": [
+                    null
+                  ]
+                }
+        :param text: 老接口的标题
+        :param title: 新接口的标题
+        :param desp: 描述信息，支持markdown
         :return:
         '''
-        res = requests.get(url=self.__api.format(self._token, text)).json()
-        if res.get('errno') == 0:
-            print("Server酱消息发送成功！")
-            return True
+
+        if self._token.startswith("SCU"):
+            # old api
+            print("你正在使用老版Server酱，该版本在21年4月底将失效，请尽快迁移到新版")
+            res = requests.get(url=self.__api.format(token=self._token, text=text, desp=desp)).json()
+            if res.get('errno') == 0:
+                print("Server酱消息发送成功！")
+                return True
+            else:
+                print("Server酱消息发送失败，原因：{}".format(res.get('errmsg')))
+                return False
         else:
-            print("Server酱消息发送失败，原因：{}".format(res.get('errmsg')))
-            return False
+            # new api
+            res = requests.get(url=self.__new_api.format(token=self._token, title=title, desp=desp)).json()
+            if res.get('code') == 0:
+                print("Server酱消息发送成功！")
+                return True
+            else:
+                print("Server酱消息发送失败，原因：{}".format(res.get('info')))
+                return False
+
+
 
 
 class DingDing(Notifier):
@@ -93,11 +155,11 @@ class DingDing(Notifier):
     """
 
     def __init__(self, token=None, *args, **kwargs):
-        super(DingDing, self).__init__(*args, **kwargs)
         self._token = token
         if self._token:
             self.__webhook = "https://oapi.dingtalk.com/robot/send?access_token={}".format(self._token)
             self._dd = DingtalkChatbot(webhook=self.__webhook)
+        super(DingDing, self).__init__(*args, **kwargs)
 
 
     @property
@@ -111,6 +173,9 @@ class DingDing(Notifier):
         self.__webhook = "https://oapi.dingtalk.com/robot/send?access_token={}".format(self._token)
         self._dd = DingtalkChatbot(webhook=self.__webhook)
 
+    def _check_configure(self):
+        if not self._token:
+            raise AuthError('请设置DingDing token')
 
     def send_text(self, text):
         '''
@@ -148,12 +213,15 @@ class EmailSender(Notifier):
         :param args:
         :param kwargs:
         '''
-        super(EmailSender, self).__init__(*args, **kwargs)
         self._user = user
         self._passwd_or_auth_code = password_or_auth_code
         self._host = host
         self.__yag = yagmail.SMTP(user=self._user, password=self._passwd_or_auth_code, host=self._host)
+        super(EmailSender, self).__init__(*args, **kwargs)
 
+    def _check_configure(self):
+        if not (self._user and self._passwd_or_auth_code and self._host):
+            raise AuthError('请完整配置EmailSender信息')
 
     def send_text(self, text, contents='', to=[]):
         '''
@@ -168,34 +236,6 @@ class EmailSender(Notifier):
             self.__yag.send(to=to, subject=text, contents=contents)
         except smtplib.SMTPAuthenticationError:
             print("send mail error, please check your password or auth code.")
-
-
-
-def test_email(cfg):
-    email = EmailSender(user=cfg.get('Email','user'),
-                        password_or_auth_code=cfg.get('Email','passwd'),
-                        host=cfg.get('Email', 'host'))
-    email.send_text(text='test', contents='this is a test mail', to=['m17713549835@163.com'])
-
-
-def test_ServerChan(cfg):
-    SJ = ServerChan()
-    SJ.token=cfg.get('ServerChan', 'token')
-    SJ.send_text(text='This is a test.')
-
-
-def test_DingDing(cfg):
-    dd = DingDing()
-    dd.token = cfg.get('DingDing', 'token')
-    dd.send_text(text='【通知】This is a test')  # 钉钉如果有关键字，自己添加测试的关键字
-
-
-from configparser import ConfigParser
-
-if __name__ == "__main__":
-    cfg = ConfigParser()
-    cfg.read('config.ini',encoding='utf-8')
-    test_email(cfg)
-    test_ServerChan(cfg)
-    test_DingDing(cfg)
-    mac_notify(title='Warning', text='This is a test')
\ No newline at end of file
+            return False
+        else:
+            return True
